@@ -2,7 +2,7 @@
 #define VIENNACL_COORDINATE_MATRIX_HPP_
 
 /* =========================================================================
-   Copyright (c) 2010-2015, Institute for Microelectronics,
+   Copyright (c) 2010-2016, Institute for Microelectronics,
                             Institute for Analysis and Scientific Computing,
                             TU Wien.
    Portions of this software are copyright by UChicago Argonne, LLC.
@@ -31,6 +31,7 @@
 
 #include "viennacl/linalg/sparse_matrix_operations.hpp"
 
+#include <Rcpp.h>
 namespace viennacl
 {
 
@@ -91,9 +92,9 @@ void copy(const CPUMatrixT & cpu_matrix,
     group_boundaries.set(group_num, data_index);
     //group_boundaries[1] = data_index; //for one compute unit
 
-    //std::cout << "Group boundaries: " << std::endl;
+    //Rcpp::Rcout << "Group boundaries: " << std::endl;
     //for (vcl_size_t i=0; i<group_boundaries.size(); ++i)
-    //  std::cout << group_boundaries[i] << std::endl;
+    //  Rcpp::Rcout << group_boundaries[i] << std::endl;
 
     viennacl::backend::memory_create(gpu_matrix.group_boundaries_, group_boundaries.raw_size(), traits::context(gpu_matrix.group_boundaries_), group_boundaries.get());
     viennacl::backend::memory_create(gpu_matrix.coord_buffer_,         coord_buffer.raw_size(), traits::context(gpu_matrix.coord_buffer_),     coord_buffer.get());
@@ -110,7 +111,14 @@ template<typename NumericT, unsigned int AlignmentV>
 void copy(const std::vector< std::map<unsigned int, NumericT> > & cpu_matrix,
           coordinate_matrix<NumericT, AlignmentV> & gpu_matrix )
 {
-  copy(tools::const_sparse_matrix_adapter<NumericT>(cpu_matrix, cpu_matrix.size(), cpu_matrix.size()), gpu_matrix);
+  vcl_size_t max_col = 0;
+  for (vcl_size_t i=0; i<cpu_matrix.size(); ++i)
+  {
+    if (cpu_matrix[i].size() > 0)
+      max_col = std::max<vcl_size_t>(max_col, (cpu_matrix[i].rbegin())->first);
+  }
+
+  viennacl::copy(tools::const_sparse_matrix_adapter<NumericT>(cpu_matrix, cpu_matrix.size(), max_col + 1), gpu_matrix);
 }
 
 //gpu to cpu:
@@ -136,7 +144,7 @@ void copy(const coordinate_matrix<NumericT, AlignmentV> & gpu_matrix,
     viennacl::backend::typesafe_host_array<unsigned int> coord_buffer(gpu_matrix.handle12(), 2*gpu_matrix.nnz());
     std::vector<NumericT> elements(gpu_matrix.nnz());
 
-    //std::cout << "GPU nonzeros: " << gpu_matrix.nnz() << std::endl;
+    //Rcpp::Rcout << "GPU nonzeros: " << gpu_matrix.nnz() << std::endl;
 
     viennacl::backend::memory_read(gpu_matrix.handle12(), 0, coord_buffer.raw_size(), coord_buffer.get());
     viennacl::backend::memory_read(gpu_matrix.handle(),   0, sizeof(NumericT) * elements.size(), &(elements[0]));
@@ -157,6 +165,11 @@ template<typename NumericT, unsigned int AlignmentV>
 void copy(const coordinate_matrix<NumericT, AlignmentV> & gpu_matrix,
           std::vector< std::map<unsigned int, NumericT> > & cpu_matrix)
 {
+  if (cpu_matrix.size() == 0)
+    cpu_matrix.resize(gpu_matrix.size1());
+
+  assert(cpu_matrix.size() == gpu_matrix.size1() && bool("Matrix dimension mismatch!"));
+
   tools::sparse_matrix_adapter<NumericT> temp(cpu_matrix, gpu_matrix.size1(), gpu_matrix.size2());
   copy(gpu_matrix, temp);
 }
@@ -297,7 +310,7 @@ public:
 
       stl_sparse_matrix.resize(new_size1);
 
-      //std::cout << "Cropping STL matrix of size " << stl_sparse_matrix.size() << std::endl;
+      //Rcpp::Rcout << "Cropping STL matrix of size " << stl_sparse_matrix.size() << std::endl;
       if (new_size2 < cols_ && rows_ > 0)
       {
         for (vcl_size_t i=0; i<stl_sparse_matrix.size(); ++i)
@@ -314,7 +327,7 @@ public:
           for (std::list<unsigned int>::iterator it = to_delete.begin(); it != to_delete.end(); ++it)
             stl_sparse_matrix[i].erase(*it);
         }
-        //std::cout << "Cropping done..." << std::endl;
+        //Rcpp::Rcout << "Cropping done..." << std::endl;
       }
 
       rows_ = new_size1;
@@ -405,11 +418,11 @@ namespace detail
       if (viennacl::traits::handle(lhs) == viennacl::traits::handle(rhs.rhs()))
       {
         viennacl::vector<T> temp(lhs);
-        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), temp);
+        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), T(1), temp, T(0));
         lhs = temp;
       }
       else
-        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), lhs);
+        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), T(1), lhs, T(0));
     }
   };
 
@@ -418,9 +431,15 @@ namespace detail
   {
     static void apply(vector_base<T> & lhs, vector_expression<const coordinate_matrix<T, A>, const vector_base<T>, op_prod> const & rhs)
     {
-      viennacl::vector<T> temp(lhs);
-      viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), temp);
-      lhs += temp;
+      // check for the special case x += A * x
+      if (viennacl::traits::handle(lhs) == viennacl::traits::handle(rhs.rhs()))
+      {
+        viennacl::vector<T> temp(lhs);
+        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), T(1), temp, T(0));
+        lhs += temp;
+      }
+      else
+        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), T(1), lhs, T(1));
     }
   };
 
@@ -429,9 +448,15 @@ namespace detail
   {
     static void apply(vector_base<T> & lhs, vector_expression<const coordinate_matrix<T, A>, const vector_base<T>, op_prod> const & rhs)
     {
-      viennacl::vector<T> temp(lhs);
-      viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), temp);
-      lhs -= temp;
+      // check for the special case x -= A * x
+      if (viennacl::traits::handle(lhs) == viennacl::traits::handle(rhs.rhs()))
+      {
+        viennacl::vector<T> temp(lhs);
+        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), T(1), temp, T(0));
+        lhs -= temp;
+      }
+      else
+        viennacl::linalg::prod_impl(rhs.lhs(), rhs.rhs(), T(-1), lhs, T(1));
     }
   };
 
